import { Label, Label2, LabelText, MySprite, ShapeRect, tweenChange } from "./Unit";

export class BigCard{
    data;

    images;

    scaleX = 1;

    scaleY = 1;

    bg : ShapeRect;

    closePic : MySprite;

    shown = false;

    contentSpr;

    init(images, scaleX = 1, scaleY = 1) {

        this.images = images;

        this.scaleX = scaleX;

        this.scaleY = scaleY;

        this.initView();
    }

    private initView(){
        
        let w = 1040 * this.scaleX;

        let h = 640 * this.scaleY

        const bg = new ShapeRect();
        bg.setSize(w, h);
        this.bg = bg;
        bg.alpha = 0;
        this.initCard();

        this.closePic = this.getColsePic(this.bg);

        this.bg.setScaleXY(0);
    }

    private initCard(){
        const card = new MySprite();
        card.init(this.images.get('card'));
        card.setScaleXY(this.scaleX);

        card.x = this.bg.width / 2;
        card.y = this.bg.height / 2;

        this.bg.addChild(card);


        
    }

    private getColsePic(parent) {
        const closePic = new MySprite();
        closePic.init(this.images.get('close'));
        closePic.scaleX = this.scaleX;
        closePic.scaleY = this.scaleY;

        closePic.x = parent.width - 20 * this.scaleX;

        closePic.y = 20 * this.scaleY;
        parent.addChild(closePic, 99);

        return closePic;
    }

    setContent(data){

        this.shown = false;

        if(this.contentSpr){
            this.bg.removeChild(this.contentSpr);
        }

        this.data = data;

        let mW = 960 * this.scaleX;

        let mH = 560 * this.scaleY;

        const pic = new MySprite();
        pic.init(this.images.get(this.data.picUrl));
        pic.x = this.bg.width / 2;
        pic.y = this.bg.height / 2 - 10 * this.scaleY;

        let scale = Math.min(mW / pic.width, mH / pic.height);
        pic.setScaleXY(scale);
        this.contentSpr = pic;
        this.bg.addChild(pic, 98);
    }

    showCard(toX, toY, callback = null){
        this.shown = true;

        tweenChange(this.bg, {scaleX: 1, scaleY: 1, x : toX, y : toY}, 0.3, callback);
    }

    hideCard(toX, toY, callback = null){
        

        tweenChange(this.bg, { scaleX: 0, scaleY: 0, x: toX, y: toY }, 0.3, () => {

            this.shown = false;

            callback && callback();
        });
    }
}

export class WordBigCard extends BigCard{

    picBgArr = [];

    wordArr = [];

    setContent(data){

        this.data = data;

        this.shown = false;

        let len = this.data.cardWordArr.length;

        let width = 900 * this.scaleX;

        let maxW = 700 * this.scaleX;

        let lineW = 400 * this.scaleX;

        let totalW = width;

        let dW = 31 * this.scaleX;

        if(len == 2){
            width = 440 * this.scaleX;
            maxW = 360 * this.scaleX;
            lineW = 180 * this.scaleX;

            totalW = width * 2 + dW;
        }

        let x = (this.bg.width - totalW ) / 2;

        for(let i = 0; i < len ; ++ i){

            let cardWord = this.data.cardWordArr[i];

            let picBg = this.getView(cardWord, width, maxW, lineW);
            picBg.x = x;
            picBg.y = (this.bg.height - picBg.height) / 2 - 10 * this.scaleY;
            picBg['data'] = cardWord;

            this.picBgArr.push(picBg);
            x += width + dW; 

            this.bg.addChild(picBg);
        }
    }

    private getView(cardWord, width, maxW, lineW){

        let height = 514 * this.scaleY;
        const picBg = new ShapeRect();
        picBg.setSize(width, height);
        picBg.fillColor = "#f7effc";

        let maxH = 300 * this.scaleY;

        const pic = new MySprite();
        pic.init(this.images.get(cardWord.picUrl));
        let scale = Math.min(maxW / pic.width, maxH / pic.height);
        pic.setScaleXY(scale);
        picBg.addChild(pic);
        pic.x = width / 2;

        if (cardWord.letterArr && cardWord.letterArr.length > 0) {
            pic.y = (height - 140 * this.scaleY) / 2 ;

            const line = new ShapeRect();
            line.setSize(lineW, 7 * this.scaleY);
            line.fillColor = "#949494";
            line.x = (width - line.width) / 2;
            line.y = height - line.height - 42 * this.scaleY;
            picBg.addChild(line);
            picBg['line'] = line;



            let text2 = this.getTextView(cardWord.letterArr);
            text2.y = height / 2 - 195 * this.scaleY + 325 * this.scaleY + text2.getBoundingBox().height / 2;
            text2.x = (width - text2.getBoundingBox().width) / 2;
            text2.alpha = 0;

            let text = this.getTextView2(cardWord.letterArr, text2.scaleX);
            text.x = text2.x;
            text.y = height / 2 - 195 * this.scaleY + 325 * this.scaleY;
            text.alpha = 0;
            
            picBg.addChild(text);
            picBg.addChild(text2);
            picBg['text'] = text;
            picBg['text2'] = text2;
        }
        else {
            pic.y = height / 2;
        }

        return picBg;
    }

    getTextView(letterArr) {

        const getLabelText = (letter) => {
            const text = new LabelText();
            text.val = letter.val;
            text.fontColor = letter.fontColor;
            text.underLine = false;

            return text;
        }

        let labelTextArr = [];

        for (let i = 0; i < letterArr.length; ++i) {

            var letter = letterArr[i];

            let label = getLabelText(letter);

            labelTextArr.push(label);
        }

        const answer = new Label2();
        answer.textArr = labelTextArr;
        answer.fontSize = 88;
        answer.fontName = "MMB";
        answer.setScaleXY(this.scaleX);
        answer.refreshSize();
        answer.setMaxSize(360 * this.scaleX);
        return answer;
    }

    getTextView2(letterArr, scale) {

        let getLabel = (letter, fontColor) => {
            const label = new Label();

            label.text = letter;
            label.textAlign = 'center';
            label.fontSize = 88;
            label.fontName = "MMB";
            label.fontColor = fontColor;
            label.setScaleXY(scale);
            label.refreshSize();
            label.y = label.getBoundingBox().height / 2;
            return label;
        }

        const bg = new ShapeRect();

        let h = 0;

        let w = 0;

        let x = 0;

        let tmpArr = [];

        for (let i = 0; i < letterArr.length; ++i) {

            let letter = letterArr[i];

            if (!letter) {
                continue;
            }

            let fontColor = letter.fontColor;

            for (let j = 0; j < letter.val.length; ++j) {
                let label = getLabel(letter.val.charAt(j), fontColor);
                bg.addChild(label);
                tmpArr.push(label);
            }
        }

        for (let i = 0; i < tmpArr.length; ++i) {
            let label = tmpArr[i];
            label.x = x;
            let tW = Math.ceil(label.getBoundingBox().width);
            x += tW;
            h = label.getBoundingBox().height;
            w += tW;
        }

        bg.alpha = 0;
        bg.visible = false;
        bg.setSize(w, h);

        return bg;
    }
}