import { BigCard, WordBigCard } from "./BigCard";
import { jelly, Label2, LabelText, MySprite, ShapeRect, ShapeRectNew, tweenChange } from "./Unit";

export class Exercises {
    images;

    data;

    scaleX = 1;

    scaleY = 1;

    width = 458;

    height = 289;

    bg;

    wordArr = []

    wordBgArr = []

    pic;

    bigCard : BigCard;

    wordBigCardArr = [];

    constructor(data, images) {
        this.data = data;

        this.images = images;
    }

    init(scaleX = 1, scaleY = 1) {
        this.scaleX = scaleX;

        this.scaleY = scaleY;

        this.width *= this.scaleX;

        this.height *= this.scaleY;

        this.initBg();

        this.initPic();

        this.initBigCard();

        this.initWord();

        this.initCardWord();
    }

    private initBg() {
        const bg = new ShapeRect();
        bg.setSize(this.width, this.height);
        bg.fillColor = "#ffffff";
        bg.setShadow(0, 2, 4, 'rgba(0, 0, 0, 0.5)');
        this.bg = bg;
    }

    private initPic() {
        let maxW = 458 * this.scaleX;

        let maxH = 246 * this.scaleY;

        const pic = new MySprite();
        pic.init(this.images.get(this.data.picUrl));
        pic.x = this.width / 2;
        pic.y = maxH / 2;
        this.bg.addChild(pic);
        let scale = Math.min(maxW / pic.width, maxH / pic.height);

        pic.setScaleXY(scale);

        this.pic = pic;
    }

    private initWord() {

        let wordArr = this.data.wordArr;

        let len = wordArr.length;

        let h = 42 * this.scaleY;

        let w = this.width;

        if (len == 2) {
            w = 229 * this.scaleX;
        }

        let x = 0;

        for (let i = 0; i < len; ++i) {
            const wordBg = new ShapeRect();
            wordBg.setSize(w, h);

            let flag = "";

            if (i == 0) {
                wordBg.fillColor = "#ecd1fa";
                flag = 'l';
            }
            else {
                wordBg.fillColor = "#f5dc6f";
                flag = 'r';
            }
            wordBg.y = this.bg.height - h;
            wordBg.x = x;

            x += (i + 1) * w;

            this.bg.addChild(wordBg);

            this.wordBgArr.push(wordBg);

            this.getWord(wordBg, wordArr[i], flag);
        }

    }

    private getWord(parent, val, flag) {
        let text = new Label2();
        text.fontName = "GOTHICB";
        let fontSize = 30;
        text.fontSize = fontSize;

        let textArr = new Array<LabelText>();
        text.textArr = textArr;

        for (let i = 0; i < val.letterArr.length; ++i) {

            var letter = val.letterArr[i];

            let labelText = new LabelText();
            labelText.val = letter.val;

            labelText.fontColor = letter.fontColor;

            textArr.push(labelText);
        }

        text.scaleX = this.scaleX;
        text.scaleY = this.scaleY;
        text.refreshSize();
        text.y = parent.height / 2;

        if('l' == flag){
            text.x = 14 * this.scaleX;
        }
        else{
            text.x = parent.width - text.getBoundingBox().width - 10 * this.scaleX;
        }

        parent.addChild(text);

        this.wordArr.push(text);
    }

    private initBigCard(){

        var data = {
            picUrl : this.data.picUrl2,
            audioUrl : this.data.audioUrl
        }

        const bigCard = new BigCard();
        bigCard.init(this.images, this.scaleX, this.scaleY);
        this.bigCard = bigCard;
        bigCard.setContent(data);
        bigCard['type'] = 'picCard';
    }

    private initCardWord(){

        for(let i = 0; i < this.data.wordArr.length; ++ i){
            let data = this.data.wordArr[i];
            const bigCard = new WordBigCard();
            bigCard.init(this.images, this.scaleX, this.scaleY);
            this.wordBigCardArr.push(bigCard);
            bigCard.setContent(data);
            bigCard['type'] = 'wordCard';
        }
        
    }

    textAnimal(item, callback = null) {

        let labelArr = item.text.children;

        const animal = (label, index) => {


            tweenChange(label, { y: label.y - 12 * this.scaleY }, 0.08, () => {

                let i = index + 1;

                if (labelArr[i]) {
                    animal(labelArr[i], i);
                }

                tweenChange(label, { y: label.y + 16 * this.scaleY }, 0.08, () => {
                    tweenChange(label, { y: label.y - 4 * this.scaleY }, 0.04, () => {
                        if (callback) {
                            callback();

                            if (index == labelArr.length - 1) {
                                item.text2.alpha = 1;
                                item.text.visible = false;
                                jelly(item.text2, 0.7)
                            }
                        }
                    });
                });
            });
        }

        item.text2.alpha = 0;
        item.text.visible = true;
        animal(labelArr[0], 0);

    }
}