import { Component, ElementRef, ViewChild, OnInit, OnDestroy, HostListener } from '@angular/core';

import {
  ShapeRect,

} from './Unit';
import { res, resAudio } from './resources';

import { Subject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';

import TWEEN from '@tweenjs/tween.js';
import { Listening } from './Listening';
import { Title } from './Title';
import { getDefaultCardWord, getDefaultExercises, getDefaultWord } from '../bean/ExercisesBean';
import { getDefaultTile } from '../bean/TitleBean';
import { Letter } from './Letter';
import { Exercises } from './Exercises';
import { defaultVal } from './defaultValue.js';




@Component({
  selector: 'app-play',
  templateUrl: './play.component.html',
  styleUrls: ['./play.component.css']
})
export class PlayComponent implements OnInit, OnDestroy {

  @ViewChild('canvas', { static: true }) canvas: ElementRef;
  @ViewChild('wrap', { static: true }) wrap: ElementRef;

  // 数据
  data;

  ctx;

  canvasWidth = 1280; // canvas实际宽度
  canvasHeight = 720; // canvas实际高度

  canvasBaseW = 1280; // canvas 资源预设宽度
  canvasBaseH = 720;  // canvas 资源预设高度

  mx; // 点击x坐标
  my; // 点击y坐标


  // 资源
  rawImages = new Map(res);
  rawAudios = new Map(resAudio);

  images = new Map();

  animationId: any;
  winResizeEventStream = new Subject();

  audioObj = {};

  renderArr;
  bigCardRenderArr = [];
  mapScale = 1;

  canvasLeft;
  canvasTop;

  saveKey = 'YM5-22';

  /**标题*/
  title;

  listening;

  curAudio;

  canTouch = true;

  letterArr = [];

  exercisesArr = [];

  curBigCard;

  curExercises;

  curWord;

  mask: ShapeRect;

  @HostListener('window:resize', ['$event'])
  onResize(event) {
    this.winResizeEventStream.next();
  }


  ngOnInit() {

    this.data = {};

    // 获取数据
    const getData = (<any>window).courseware.getData;
    getData((data) => {

      if (data && typeof data == 'object') {
        this.data = data;
      }
      else{
        this.data = defaultVal;
      }
      //console.log('data:', JSON.stringify(data));

      // 初始化 各事件监听
      this.initListener();

      // 若无数据 则为预览模式 需要填充一些默认数据用来显示
      this.initDefaultData();

      // 初始化 音频资源
      this.initAudio();
      // 初始化 图片资源
      this.initImg();
      // 开始预加载资源
      this.load();

    }, this.saveKey);

  }

  ngOnDestroy() {
    window['curCtx'] = null;
    window.cancelAnimationFrame(this.animationId);
    this.cleanAudio();
  }


  cleanAudio() {
    if (this.audioObj) {
      for (const key in this.audioObj) {
        this.audioObj[key].pause();
      }
    }
  }


  load() {

    // 预加载资源
    this.loadResources().then(() => {
      window["air"].hideAirClassLoading(this.saveKey, this.data);
      this.init();
      this.update();
    });
  }


  init() {

    this.initCtx();
    this.initData();
    this.initView();
  }

  initCtx() {
    this.canvasWidth = this.wrap.nativeElement.clientWidth;
    this.canvasHeight = this.wrap.nativeElement.clientHeight;
    this.canvas.nativeElement.width = this.wrap.nativeElement.clientWidth;
    this.canvas.nativeElement.height = this.wrap.nativeElement.clientHeight;


    this.ctx = this.canvas.nativeElement.getContext('2d');
    this.canvas.nativeElement.width = this.canvasWidth;
    this.canvas.nativeElement.height = this.canvasHeight;

    window['curCtx'] = this.ctx;
    window['curImages'] = this.images;
  }






  updateItem(item) {
    if (item) {
      item.update();
    }
  }

  updateArr(arr) {
    if (!arr) {
      return;
    }
    for (let i = 0; i < arr.length; i++) {
      arr[i].update(this);
    }
  }







  initListener() {

    this.winResizeEventStream
      .pipe(debounceTime(500))
      .subscribe(data => {
        this.renderAfterResize();
      });


    // ---------------------------------------------
    const setParentOffset = () => {
      const rect = this.canvas.nativeElement.getBoundingClientRect();
      this.canvasLeft = rect.left;
      this.canvasTop = rect.top;
    };
    const setMxMyByTouch = (event) => {
      if (event.touches.length <= 0) {
        return;
      }
      if (this.canvasLeft == null) {
        setParentOffset();
      }
      this.mx = event.touches[0].pageX - this.canvasLeft;
      this.my = event.touches[0].pageY - this.canvasTop;
    };

    const setMxMyByMouse = (event) => {
      this.mx = event.offsetX;
      this.my = event.offsetY;
    };
    // ---------------------------------------------


    let firstTouch = true;

    const touchDownFunc = (e) => {
      if (firstTouch) {
        firstTouch = false;
        removeMouseListener();
      }
      setMxMyByTouch(e);
      this.mapDown(e);
    };
    const touchMoveFunc = (e) => {
      setMxMyByTouch(e);
      this.mapMove(e);
    };
    const touchUpFunc = (e) => {
      setMxMyByTouch(e);
      this.mapUp(e);
    };

    const mouseDownFunc = (e) => {
      if (firstTouch) {
        firstTouch = false;
        removeTouchListener();
      }
      setMxMyByMouse(e);
      this.mapDown(e);
    };
    const mouseMoveFunc = (e) => {
      setMxMyByMouse(e);
      this.mapMove(e);
    };
    const mouseUpFunc = (e) => {
      setMxMyByMouse(e);
      this.mapUp(e);
    };


    const element = this.canvas.nativeElement;

    const addTouchListener = () => {
      element.addEventListener('touchstart', touchDownFunc);
      element.addEventListener('touchmove', touchMoveFunc);
      element.addEventListener('touchend', touchUpFunc);
      element.addEventListener('touchcancel', touchUpFunc);
    };
    const removeTouchListener = () => {
      element.removeEventListener('touchstart', touchDownFunc);
      element.removeEventListener('touchmove', touchMoveFunc);
      element.removeEventListener('touchend', touchUpFunc);
      element.removeEventListener('touchcancel', touchUpFunc);
    };

    const addMouseListener = () => {
      element.addEventListener('mousedown', mouseDownFunc);
      element.addEventListener('mousemove', mouseMoveFunc);
      element.addEventListener('mouseup', mouseUpFunc);
    };
    const removeMouseListener = () => {
      element.removeEventListener('mousedown', mouseDownFunc);
      element.removeEventListener('mousemove', mouseMoveFunc);
      element.removeEventListener('mouseup', mouseUpFunc);
    };

    addMouseListener();
    addTouchListener();
  }


  playAudio(key, now = false, callback = null) {

    const audio = this.audioObj[key];
    if (audio) {

      this.pauseAudio(this.curAudio);

      if (now) {
        audio.pause();
        audio.currentTime = 0;
      }

      if (callback) {
        audio.onended = () => {
          callback();
        };
      }
      audio.play();

    }

    return audio;
  }

  pauseAudio(audio) {
    if (audio) {
      if (this.listening && audio == this.listening.audio) {
        this.listening.pauseAudio();
      }
      else {
        audio.pause();
      }
    }
  }



  loadResources() {
    const pr = [];
    this.rawImages.forEach((value, key) => {// 预加载图片

      const p = this.preload(value)
        .then(img => {
          this.images.set(key, img);
        })
        .catch(err => console.log(err));

      pr.push(p);
    });

    this.rawAudios.forEach((value, key) => {// 预加载音频

      const a = this.preloadAudio(value)
        .then(() => {
          // this.images.set(key, img);
        })
        .catch(err => console.log(err));

      pr.push(a);
    });
    return Promise.all(pr);
  }

  preload(url) {
    return new Promise((resolve, reject) => {
      const img = new Image();
      // img.crossOrigin = "anonymous";
      img.onload = () => resolve(img);
      img.onerror = reject;
      img.src = url;
    });
  }

  preloadAudio(url) {
    return new Promise((resolve, reject) => {
      const audio = new Audio();
      audio.oncanplay = (a) => {
        resolve(a);
      };
      audio.onerror = () => {
        reject();
      };
      audio.src = url;
      audio.load();
    });
  }


  renderAfterResize() {
    this.canvasWidth = this.wrap.nativeElement.clientWidth;
    this.canvasHeight = this.wrap.nativeElement.clientHeight;
    this.init();
  }





  checkClickTarget(target) {

    const rect = target.getBoundingBox();

    if (this.checkPointInRect(this.mx, this.my, rect)) {
      return true;
    }
    return false;
  }

  getWorlRect(target) {

    let rect = target.getBoundingBox();

    if (target.parent) {

      const pRect = this.getWorlRect(target.parent);
      rect.x += pRect.x;
      rect.y += pRect.y;
    }
    return rect;
  }

  checkPointInRect(x, y, rect) {
    if (x >= rect.x && x <= rect.x + rect.width) {
      if (y >= rect.y && y <= rect.y + rect.height) {
        return true;
      }
    }
    return false;
  }





  addUrlToAudioObj(key, url = null, vlomue = 1, loop = false, callback = null) {

    const audioObj = this.audioObj;

    if (url == null) {
      url = key;
    }

    this.rawAudios.set(key, url);

    const audio = new Audio();
    audio.src = url;
    audio.load();
    audio.loop = loop;
    audio.volume = vlomue;

    audioObj[key] = audio;

    if (callback) {
      audio.onended = () => {
        callback();
      };
    }
  }

  addUrlToImages(url) {
    this.rawImages.set(url, url);
  }






  // ======================================================编写区域==========================================================================





  /**
   * 添加默认数据 便于无数据时的展示
   */
  initDefaultData() {

    if (!this.data.title) {
      this.data.title = getDefaultTile();
    }

    if (!this.data.letterArr) {
      this.data.letterArr = [];
    }

    if (!this.data.exercisesArr) {
      this.data.exercisesArr = [];
    }

    for (let i = this.data.exercisesArr.length; i < 4; ++i) {

      let exercises = getDefaultExercises();

      this.data.exercisesArr.push(exercises);

    }

    for (let i = 0; i < this.data.exercisesArr.length; ++i) {

      let exercises = this.data.exercisesArr[i];

      if (!exercises.wordArr) {
        exercises.wordArr = [];
      }

      if (exercises.wordArr.length == 0) {
        let word = getDefaultWord();
        exercises.wordArr.push(word);
      }

      for (let j = 0; j < exercises.wordArr.length; ++j) {

        let word = exercises.wordArr[j];

        if (!word.cardWordArr) {
          word.cardWordArr = [];
        }

        if (word.cardWordArr.length == 0) {
          let cardWord = getDefaultCardWord();
          word.cardWordArr.push(cardWord);
        }
      }
    }

  }


  /**
   * 添加预加载图片
   */
  initImg() {
    for (let i = 0; i < this.data.exercisesArr.length; ++i) {

      let exercises = this.data.exercisesArr[i];

      if (exercises.picUrl) {
        this.addUrlToImages(exercises.picUrl);
      }

      if (exercises.picUrl2) {
        this.addUrlToImages(exercises.picUrl2);
      }

      for (let j = 0; j < exercises.wordArr.length; ++j) {
        let word = exercises.wordArr[j];

        for (let m = 0; m < word.cardWordArr.length; ++m) {

          let cardWord = word.cardWordArr[m];

          if (cardWord.picUrl) {
            this.addUrlToImages(cardWord.picUrl);
          }
        }

      }
    }


  }

  /**
   * 添加预加载音频
   */
  initAudio() {

    // 音效
    this.rawAudios.forEach((item, key) => {
      // 音效
      this.addUrlToAudioObj(key, item);
    })

    //标题发音
    if (this.data.title.audioUrl) {
      this.addUrlToAudioObj('titleAudio', this.data.title.audioUrl);
    }


    //听力材料
    if (this.data.listenAudioUrl) {
      this.addUrlToAudioObj('listenAudio', this.data.listenAudioUrl);
    }

    for (let i = 0; i < this.data.letterArr.length; ++i) {
      let letter = this.data.letterArr[i];

      if (letter.audioUrl) {
        this.addUrlToAudioObj(letter.audioUrl);
      }
    }

    for (let i = 0; i < this.data.exercisesArr.length; ++i) {

      let exercises = this.data.exercisesArr[i];

      if (exercises.audioUrl) {
        this.addUrlToAudioObj(exercises.audioUrl);
      }

      for (let j = 0; j < exercises.wordArr.length; ++j) {
        let word = exercises.wordArr[j];

        for (let m = 0; m < word.cardWordArr.length; ++m) {
          let cardWord = word.cardWordArr[m];
          if (cardWord.audioUrl) {
            this.addUrlToAudioObj(cardWord.audioUrl);
          }
        }
      }


    }
  }



  /**
   * 初始化数据
   */
  initData() {

    const sx = this.canvasWidth / this.canvasBaseW;
    const sy = this.canvasHeight / this.canvasBaseH;
    const s = Math.min(sx, sy);
    this.mapScale = s;


    this.renderArr = [];

    this.bigCardRenderArr = [];

    this.letterArr = [];

    this.exercisesArr = [];

    this.curBigCard = null;
  }



  /**
   * 初始化试图
   */
  initView() {

    this.initTitle();

    this.initLetter();

    this.initBottomPart();

    this.initExercises();

    this.initMask();
  }

  initBottomPart() {

    if (!this.data.listenAudioUrl) {
      return;
    }

    const listening = new Listening(this.audioObj['listenAudio'], this.images);
    listening.init(this.mapScale, this.mapScale);

    listening.bg.x = this.canvasWidth - listening.bg.width - 10 * this.mapScale;
    listening.bg.y = this.canvasHeight - listening.bg.height - 6.5 * this.mapScale;
    this.listening = listening;

    this.renderArr.push(listening.bg);
  }

  initTitle() {
    const title = new Title(this.data.title, this.images);

    title.init(this.mapScale, this.mapScale);
    title.audio = this.audioObj['titleAudio'];
    this.title = title;

    title.titleBg.y = 0;
    title.titleBg.x = 0;

    this.renderArr.push(title.titleBg);
  }

  initLetter() {

    let x = this.canvasWidth / 2;

    let dw = 17 * this.mapScale;

    for (let i = 0; i < this.data.letterArr.length; ++i) {

      let data = this.data.letterArr[i];

      let letter = new Letter();
      letter.init(this.images, data, '#763c92', this.mapScale, this.mapScale);

      letter.bg.y = 0;
      letter.bg.x = x;

      x += letter.bg.width + dw;

      this.renderArr.push(letter.bg);

      this.letterArr.push(letter);
    }

  }

  initExercises() {

    let dw = 40 * this.mapScale;

    let dh = 19 * this.mapScale;

    let width = 458 * this.mapScale;

    let col = 2;

    let tW = col * width + (col - 1) * dw;

    let x = (this.canvasWidth - tW) / 2;
    let y = this.canvasHeight / 2 - 255 * this.mapScale;

    let tempX = x;

    for (let i = 0; i < this.data.exercisesArr.length; ++i) {
      let data = this.data.exercisesArr[i];

      let exercises = new Exercises(data, this.images);
      exercises.init(this.mapScale, this.mapScale);
      exercises.bg.x = x;
      exercises.bg.y = y;

      if ((i + 1) % 2 == 0) {

        //到下一行恢复起始x坐标
        x = tempX;

        y += exercises.bg.height + dh;

      }
      else {
        x += exercises.bg.width + dw;
      }

      this.renderArr.push(exercises.bg);

      this.exercisesArr.push(exercises);
    }
  }

  initMask() {
    const mask = new ShapeRect();
    mask.setSize(this.canvasWidth, this.canvasHeight);
    mask.visible = false;
    mask.alpha = 0.3
    mask.fillColor = "#000000";
    this.renderArr.push(mask);
    this.mask = mask;
  }


  mapDown(event) {

    if (!this.canTouch) {
      return;
    }

    //卡片显示
    if (this.curBigCard) {

      if (this.checkClickTarget(this.curBigCard.bg)) {
        this.clickCard();
        return;
      }

      return;
    }


    if (this.listening && this.checkClickTarget(this.listening.bg)) {

      let tempAudio = this.curAudio;

      if (tempAudio) {
        tempAudio.pause();
      }

      this.curAudio = this.listening.play(() => {
        this.listening.reset();
      });

      return;
    }

    if (this.checkClickTarget(this.title.titleBg)) {
      this.pauseAudio(this.curAudio);
      let audio = this.title.playAudio();
      if (audio) {
        this.curAudio = audio;
      }
      return;
    }

    for (let i = 0; i < this.letterArr.length; ++i) {
      let letter = this.letterArr[i];
      if (this.checkClickTarget(letter.bg)) {
        this.playAudio(letter.data.audioUrl);
        return;
      }
    }

    let clicked = false;

    let target;

    let callback;

    for (let i = 0; i < this.exercisesArr.length; ++i) {

      let exer = this.exercisesArr[i];

      if (this.checkClickTarget(exer.bg)) {
        callback = this.clickExercises.bind(this);
        clicked = true;
        target = exer;
        break;
      }
    }

    if (clicked) {
      this.curExercises = target;
      callback(target);
      return;
    }
  }

  mapMove(event) {

  }

  mapUp(event) {

  }



  update() {

    // ----------------------------------------------------------
    this.animationId = window.requestAnimationFrame(this.update.bind(this));
    // 清除画布内容
    this.ctx.clearRect(0, 0, this.canvasWidth, this.canvasHeight);
    // tween 更新动画
    TWEEN.update();
    // ----------------------------------------------------------



    this.updateArr(this.renderArr);

    this.updateArr(this.bigCardRenderArr);
  }


  clickExercises(item) {

    if (this.checkClickTarget(item.pic)) {

      let bigCard = item.bigCard;

      let x = this.curExercises.bg.x;
      let y = this.curExercises.bg.y;

      this.showBigCard(bigCard, x, y, () => {
        this.canTouch = true;
      })

      return;
    }
    else {
      for (let i = 0; i < item.wordBgArr.length; ++i) {

        let wordBg = item.wordBgArr[i];

        let word = item.wordArr[i]

        this.curWord = word;

        if (this.checkClickTarget(wordBg)) {

          let bigCard = item.wordBigCardArr[i];
          let x = word.getBoundingBox().x;
          let y = word.getBoundingBox().y;

          this.showBigCard(bigCard, x, y, () => {
            this.canTouch = true;
          })

          return;
        }

      }
    }
  }

  private showBigCard(bigCard, x, y, callback) {
    const index = this.bigCardRenderArr.indexOf(bigCard.bg);
    if (index == -1) {
      bigCard.bg.x = x;
      bigCard.bg.y = y;
      this.bigCardRenderArr.push(bigCard.bg);
    }

    this.playAudio('open', true);

    this.mask.visible = true;

    this.curBigCard = bigCard;

    let toX = (this.canvasWidth - bigCard.bg.width) / 2;
    let toY = (this.canvasHeight - bigCard.bg.height) / 2;

    this.canTouch = false;

    bigCard.showCard(toX, toY, callback);
  }

  clickCard() {

    let bigCard = this.curBigCard;

    if (bigCard['type'] == 'picCard') {

      if (this.checkClickTarget(bigCard.contentSpr)) {
        this.playAudio(bigCard.data.audioUrl);
        return;
      }

      if (this.checkClickTarget(bigCard.closePic)) {

        this.playAudio('close', true);

        let toX = this.curExercises.bg.x;
        let toY = this.curExercises.bg.y;
        bigCard.hideCard(toX, toY, () => {
          this.mask.visible = false;
          this.curBigCard = null;

          this.curExercises = null;
        });
      }

    }
    else if (bigCard['type'] == 'wordCard') {
      if (this.checkClickTarget(bigCard.closePic)) {

        this.playAudio('close', true);

        let toX = this.curWord.getBoundingBox().x;
        let toY = this.curWord.getBoundingBox().y;
        bigCard.hideCard(toX, toY, () => {
          this.mask.visible = false;
          this.curBigCard = null;

          this.curExercises = null;
        });

        return;
      }

      let picBgArr = bigCard.picBgArr;
      for (let i = 0; i < picBgArr.length; ++i) {
        let picBg = picBgArr[i];

        if (this.checkClickTarget(picBg)) {
          this.canTouch = false;

          if (picBg.line) {
            if (picBg.line.alpha == 1) {
              picBg.line.alpha = 0;
              picBg.text.visible = true;
              this.curExercises.textAnimal(picBg, () => {
                this.canTouch = true;
              })

            }
            else {
              this.curExercises.textAnimal(picBg, () => {
                this.canTouch = true;
              })
              this.curAudio = this.playAudio(picBg.data.audioUrl, true);
            }
          }
          else {
            this.curAudio = this.playAudio(picBg.data.audioUrl, true);
          }
          return;
        }
      }
    }
  }
}
